/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.model.data.Article
import dk.nsp.mingraviditet.model.data.ArticleCategory
import dk.nsp.mingraviditet.model.data.ArticleCategoryWithArticleCount
import dk.nsp.mingraviditet.ui.adapterviews.IdentityIllustrationRow
import dk.nsp.mingraviditet.ui.adapterviews.LoadingState
import dk.nsp.mingraviditet.ui.adapterviews.models.SectionType
import dk.nsp.mingraviditet.ui.common.SectionHeaderAdapterView
import dk.nsp.mingraviditet.ui.main.knowledge.checklist.ChecklistOverviewRow
import dk.nsp.mingraviditet.ui.main.knowledge.checklist.ChecklistOverviewRowData
import dk.nsp.mingraviditet.ui.main.knowledge.search.SearchNoResultsView
import dk.nsp.mingraviditet.util.android.motionlayout.MotionLayoutStateEnum
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.postSingleEvent
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.models.ArticleSearchResult
import dk.nsp.shared.network.Failed
import dk.nsp.shared.network.Success
import dk.nsp.util.hasSearchResults
import dk.nsp.util.searchResults
import dk.nsp.util.textmodel.TextModel
import dk.nsp.util.toTextModel
import kotlinx.coroutines.CancellationException
import kotlinx.coroutines.Job
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.onStart
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class KnowledgeViewModel @Inject constructor(
    private val articlesRepo: ArticlesRepository,
) : ViewModel() {

    private var lastSearchedString: String? = null
    private val searchInputDelayMs = 300L

    private val loadingState by lazy {
        listOf(LoadingState())
    }

    private val _isShowingSearch: MutableLiveData<Boolean> = MutableLiveData(false)

    var toolbarState: MotionLayoutStateEnum = MotionLayoutStateEnum.Start

    private val _onChecklistClicked: MutableLiveData<SingleEvent<Checklist>> =
        MutableLiveData()
    val onChecklistClicked: LiveData<SingleEvent<Checklist>> =
        _onChecklistClicked

    private val _onArticleCategoryClicked: MutableLiveData<SingleEvent<ArticleCategory>> =
        MutableLiveData()
    val onArticleCategoryClicked: LiveData<SingleEvent<ArticleCategory>> =
        _onArticleCategoryClicked

    private val searchResults = MutableSharedFlow<List<RecyclerAdapterView<*>>>(replay = 1)

    private val _onError: MutableLiveData<SingleEvent<TextModel>> =
        MutableLiveData()
    val onError: LiveData<SingleEvent<TextModel>> = _onError

    private val _onSearchedArticleClicked: MutableLiveData<SingleEvent<Pair<Article, Int>>> =
        MutableLiveData()
    val onSearchArticleClicked: LiveData<SingleEvent<Pair<Article, Int>>> =
        _onSearchedArticleClicked

    private var currentSearchJob: Job? = null

    private val articleCategoriesWithChecklists = articlesRepo.articleCategoriesWithCount
        .map {
            if (it.isEmpty()) {
                getArticleCategories()
                emptyList()
            } else {
                buildList<RecyclerAdapterView<*>> {
                    val categories = mapSectionsIntoCards(it)
                    add(
                        ChecklistOverviewRow(
                            data = ChecklistOverviewRowData {
                                _onChecklistClicked.postSingleEvent(it)
                            },
                        ),
                    )
                    add(SectionHeaderAdapterView(TextModel.from(R.string.subjects_title)))
                    addAll(categories)

                    if (categories != loadingState) {
                        add(IdentityIllustrationRow(SectionType.Knowledge))
                    }
                }
            }
        }
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.Lazily,
            initialValue = emptyList(),
        )

    val rowsToShow = articleCategoriesWithChecklists.combine(searchResults.onStart { emit(emptyList()) }) { standardRows, searchResults ->
        searchResults.ifEmpty {
            standardRows
        }
    }
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.Eagerly,
            initialValue = loadingState,
        )

    private fun getArticleCategories() = viewModelScope.launch {
        articlesRepo.getAllArticlesAndInsertIntoDatabase()
    }

    private fun mapSectionsIntoCards(categories: List<ArticleCategoryWithArticleCount>): List<CategoryCard> {
        return categories.map {
            CategoryCard(
                CategoryCardData.WithIconUrl(
                    it.category.name,
                    it.category.description,
                    TextModel.Companion.from(
                        R.plurals.articles_count,
                        it.articleCount,
                        TextModel.from("${it.articleCount}"),
                    ),
                    it.category.icon,
                ) {
                    _onArticleCategoryClicked.postValue(it.category.toSingleEvent())
                },
            )
        }
    }

    fun searchFor(searchTerm: String) = viewModelScope.launch {
        // To disable the search trigger when navigating back from detail views
        if (searchTerm == lastSearchedString) {
            return@launch
        }
        currentSearchJob?.cancel()
        if (searchTerm.isBlank()) {
            _isShowingSearch.value = false
            lastSearchedString = searchTerm
            // Post its own values, so we recreate the list
            searchResults.emit(emptyList())
        } else {
            _isShowingSearch.value = true
            currentSearchJob = viewModelScope.launch {
                delay(searchInputDelayMs)
                if (!this.isActive) return@launch
                searchResults.emit(loadingState)

                when (val searchRes = articlesRepo.searchArticles(searchTerm)) {
                    is Success -> {
                        lastSearchedString = searchTerm

                        if (!searchRes.value.hasSearchResults()) {
                            searchResults.emit(listOf(SearchNoResultsView()))
                            return@launch
                        }

                        val rowsToShow = mutableListOf<RecyclerAdapterView<*>>()
                        searchRes.value.forEach { searchResultForCategory ->
                            rowsToShow.add(
                                SectionHeaderAdapterView(
                                    TextModel.from(
                                        searchResultForCategory.name,
                                    ),
                                ),
                            )

                            val newRows =
                                searchResultForCategory.searchResults().map {
                                    createCardForSearchResult(it)
                                }
                            rowsToShow.addAll(newRows)
                        }

                        searchResults.emit(rowsToShow)
                    }

                    is Failed -> {
                        if (searchRes.error !is CancellationException) {
                            _onError.postValue(
                                SingleEvent(searchRes.error.message?.toTextModel() ?: TextModel.Companion.from("Kunne ikke hente søgeresultater")),
                            )
                        }
                    }
                }
            }
        }
    }

    private fun createCardForSearchResult(articleSearchResult: ArticleSearchResult): CategoryCard {
        return CategoryCard(
            CategoryCardData.WithIconUrl(
                articleSearchResult.title,
                articleSearchResult.description,
                TextModel.from(articleSearchResult.subtitle),
                articleSearchResult.iconUrl,
            ) {
                _onSearchedArticleClicked.postValue(
                    Pair(
                        articleSearchResult.article,
                        articleSearchResult.articleSectionId ?: -1,
                    ).toSingleEvent(),
                )
            },
        )
    }
}
