/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.article

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.webkit.WebChromeClient
import android.widget.FrameLayout
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.material.snackbar.Snackbar
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentKnowledgeArticleBinding
import dk.nsp.mingraviditet.extensions.onBackPressed
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.extensions.navigateSafe
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@AndroidEntryPoint
class KnowledgeArticleFragment : Fragment() {

    private val adapter by lazy {
        DelegatingListAdapter(viewLifecycleOwner)
    }

    private val viewModel by viewModels<KnowledgeArticleViewModel>()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        val binding = FragmentKnowledgeArticleBinding.inflate(inflater, container, false)

        binding.toolbar.setNavigationOnClickListener { onBackPressed() }

        binding.recyclerView.adapter = adapter

        viewModel.scrollToItem.observeIfNotHandled(viewLifecycleOwner) { position ->
            (binding.recyclerView.layoutManager as? LinearLayoutManager)?.scrollToPositionWithOffset(
                position,
                // Scroll the view to the top
                0,
            )
        }

        val fullScreenToggler = FullScreenToggler()
        viewModel.enterFullScreen.observe(viewLifecycleOwner) { singleEvent ->
            singleEvent.getIfNotHandled()?.let {
                fullScreenToggler.shownViewInFullScreen(it.view, it.callback)
            }
        }

        viewModel.leaveFullScreen.observe(viewLifecycleOwner) { singleEvent ->
            singleEvent.getIfNotHandled()?.let {
                fullScreenToggler.hideViewFromFullScreen()
            }
        }

        viewModel.goToOtherArticle.observeIfNotHandled(viewLifecycleOwner) { (otherArticle, relevantId) ->
            if (otherArticle != null) {
                findNavController().navigateSafe(
                    KnowledgeArticleFragmentDirections.actionKnowledgeArticleFragmentSelf(
                        articleId = otherArticle.id,
                        relevantId = relevantId,
                    ),
                )
            } else {
                Snackbar.make(binding.root, getString(R.string.article_internal_link_not_found), Snackbar.LENGTH_LONG).show()
            }
        }

        lifecycleScope.launch {
            viewModel.title.collectLatest {
                binding.toolbar.title = it
            }
        }

        lifecycleScope.launch {
            viewModel.articleRows.collectLatest {
                adapter.submitList(it)
            }
        }

        return binding.root
    }

    // Class that support showing a view in full screen. Used for the ChromeWebViewClient used by some of the row data
// shown in this fragment.
    private inner class FullScreenToggler {
        private var view: View? = null
        private var originalOrientation: Int = 0
        private var customViewCallback: WebChromeClient.CustomViewCallback? = null

        private fun hideSystemUI() {
            val activity = requireActivity()
            val window = activity.window
            val decorView = window.decorView as FrameLayout

            WindowCompat.setDecorFitsSystemWindows(window, false)
            WindowInsetsControllerCompat(window, decorView).let { controller ->
                controller.hide(WindowInsetsCompat.Type.systemBars())
                controller.systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
            }
        }

        private fun showSystemUI() {
            val activity = requireActivity()
            val window = activity.window
            val decorView = window.decorView as FrameLayout

            WindowCompat.setDecorFitsSystemWindows(window, true)
            WindowInsetsControllerCompat(window, decorView).show(WindowInsetsCompat.Type.systemBars())
        }

        fun shownViewInFullScreen(view: View, callback: WebChromeClient.CustomViewCallback) {
            if (this.view != null) {
                hideViewFromFullScreen()
                return
            }

            this.view = view

            val activity = requireActivity()
            val window = activity.window
            val decorView = window.decorView as FrameLayout

            originalOrientation = activity.requestedOrientation

            customViewCallback = callback

            decorView.addView(this.view, FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT))

            hideSystemUI()
        }

        fun hideViewFromFullScreen() {
            val activity = requireActivity()
            val window = activity.window
            val decorView = window.decorView as FrameLayout

            view?.let {
                decorView.removeView(it)
            }
            showSystemUI()
            activity.requestedOrientation = originalOrientation
            customViewCallback?.onCustomViewHidden()
            customViewCallback = null
            view = null
        }
    }
}
