/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.article

import android.view.View
import android.webkit.WebChromeClient
import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.asLiveData
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.mingraviditet.model.data.Article
import dk.nsp.mingraviditet.model.data.ArticleSection
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.postSingleEvent
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.network.Dispatcher
import dk.nsp.shared.network.MGDispatchers
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.flowOn
import kotlinx.coroutines.flow.mapLatest
import kotlinx.coroutines.launch
import java.time.LocalDateTime
import javax.inject.Inject

@HiltViewModel
class KnowledgeArticleViewModel @Inject constructor(
    private val savedStateHandle: SavedStateHandle,
    private val articlesRepository: ArticlesRepository,
    @Dispatcher(MGDispatchers.IO) private val ioDispatcher: CoroutineDispatcher,
) : ViewModel() {

    private val relevantId = savedStateHandle.get<Int>("relevantId") ?: -1

    private val articleId = MutableSharedFlow<Int>(replay = 1)

    private val article = articleId
        .mapLatest {
            articlesRepository.getPopulatedArticle(it)
        }.flowOn(ioDispatcher)

    val articleRows = article
        .mapLatest {
            createArticleRows(it.entity, it.content)
        }

    val title = article.mapLatest { it.entity.title }

    val scrollToItem: LiveData<SingleEvent<Int>> = MediatorLiveData<SingleEvent<Int>>().apply {
        addSource(articleRows.asLiveData(ioDispatcher)) {
            findRelevantId(it)?.let { relevantId ->
                postSingleEvent(relevantId)
            }
        }
    }

    private val _goToOtherArticle: MutableLiveData<SingleEvent<Pair<Article?, Int>>> = MutableLiveData()
    val goToOtherArticle: LiveData<SingleEvent<Pair<Article?, Int>>> get() = _goToOtherArticle

    private fun findLinkedArticleById(internalLink: String) = viewModelScope.launch {
        _goToOtherArticle.postValue(articlesRepository.getInternalArticleByLink(internalLink).toSingleEvent())
    }

    init {
        val requestedArticleId = savedStateHandle.get<Int>("articleId")
        viewModelScope.launch {
            if (requestedArticleId != null && requestedArticleId != -1) {
                articleId.emit(requestedArticleId)
            }
        }
    }

    private fun findRelevantId(adapterViews: List<RecyclerAdapterView<*>>): Int? {
        if (relevantId == -1) return null

        return adapterViews.indexOfFirst { it is ArticleParagraphCardRow && it.model.id == relevantId }
    }

    private fun createArticleRows(article: Article, sections: List<ArticleSection>): List<RecyclerAdapterView<*>> {
        val adapterViews = mutableListOf<RecyclerAdapterView<*>>()

        adapterViews.add(createExcerptRow(article.description, article.content))
        adapterViews.addAll(createParagraphCards(sections, relevantId))
        adapterViews.add(createLastUpdatedRow(article.lastUpdated))

        return adapterViews
    }

    private fun createExcerptRow(description: String, content: String?): ArticleExcerptRow =
        ArticleExcerptRow(description, content)

    private fun createParagraphCards(
        articleSections: List<ArticleSection>,
        relevantId: Int,
    ): List<ArticleParagraphCardRow> {
        return articleSections.sortedBy { it.sortOrder }.map {
            ArticleParagraphCardRow(
                model = it.apply { isExpanded = id == relevantId },
                relevantId = relevantId,
                onInternalLinkClicked = ::findLinkedArticleById,
                webChromeClientDelegate = WebChromeClientDelegateImpl(),
            )
        }
    }

    private fun createLastUpdatedRow(lastUpdatedDate: LocalDateTime): ArticleLastUpdatedDateRow {
        return ArticleLastUpdatedDateRow(lastUpdatedDate)
    }

    class EnterFullScreen(val view: View, val callback: WebChromeClient.CustomViewCallback)
    class LeaveFullScreen

    private val _enterFullScreen: MutableLiveData<SingleEvent<EnterFullScreen>> = MutableLiveData()
    val enterFullScreen: LiveData<SingleEvent<EnterFullScreen>> = _enterFullScreen

    private val _leaveFullScreen: MutableLiveData<SingleEvent<LeaveFullScreen>> = MutableLiveData()
    val leaveFullScreen: LiveData<SingleEvent<LeaveFullScreen>> = _leaveFullScreen

    private inner class WebChromeClientDelegateImpl : WebChromeClientDelegate {

        override fun onShowCustomView(view: View, callback: WebChromeClient.CustomViewCallback) {
            _enterFullScreen.postSingleEvent(EnterFullScreen(view, callback))
        }

        override fun onHideCustomView() {
            _leaveFullScreen.postSingleEvent(LeaveFullScreen())
        }
    }
}
