/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.compose

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.bumptech.glide.integration.compose.ExperimentalGlideComposeApi
import com.bumptech.glide.integration.compose.GlideImage
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.gravidcompose.component.GravidCard
import dk.nsp.mingraviditet.gravidcompose.component.GravidIcon
import dk.nsp.mingraviditet.gravidcompose.component.GravidText
import dk.nsp.mingraviditet.gravidcompose.theme.GravidTheme
import dk.nsp.mingraviditet.ui.main.common.compose.InnerCircularCard
import dk.nsp.models.ArticleSearchResult

@Composable
fun ArticleCard(
    article: ArticleSearchResult,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
) {
    ArticleCard(
        title = article.title,
        description = article.description,
        category = article.subtitle,
        iconUrl = article.iconUrl ?: "",
        modifier = modifier,
        onClick = onClick,
    )
}

@OptIn(ExperimentalGlideComposeApi::class)
@Composable
fun ArticleCard(
    title: CharSequence,
    description: CharSequence,
    category: CharSequence,
    iconUrl: String,
    modifier: Modifier = Modifier,
    onClick: () -> Unit,
) {
    GravidCard(
        modifier = modifier,
        onClick = onClick,
    ) {
        Row {
            InnerCircularCard(size = 48.dp) {
                GlideImage(
                    model = iconUrl,
                    contentDescription = null,
                    modifier = Modifier
                        .fillMaxSize()
                        .padding(8.dp),
                )
            }
            Column(
                Modifier
                    .weight(1f)
                    .padding(start = GravidTheme.dimens.largeSpacing),
                verticalArrangement = Arrangement.spacedBy(GravidTheme.dimens.trumpSpacing),
            ) {
                GravidText(
                    text = title,
                    style = GravidTheme.typography.Header4,
                )
                GravidText(
                    text = description,
                    maxLines = 2,
                    overflow = TextOverflow.Ellipsis,
                )
                Row(verticalAlignment = Alignment.CenterVertically) {
                    GravidText(
                        modifier = Modifier.weight(1f),
                        text = category,
                        style = GravidTheme.typography.Info,
                        color = GravidTheme.colorScheme.onSurface,
                    )
                    GravidIcon(painter = painterResource(id = R.drawable.ic_chevron_right))
                }
            }
        }
    }
}

@Preview
@Composable
fun ArticleCardPreview() {
    GravidTheme {
        Column(
            modifier = Modifier.fillMaxSize(),
            verticalArrangement = Arrangement.spacedBy(16.dp),
        ) {
            ArticleCard(
                title = "Covid-19",
                description = "Det er vigtigt, at du som gravid eller fødende oplyser, om du eller din partner har symptomer på coronavirus",
                category = "Vigtig information",
                iconUrl = "https://placehold.co/30x30",
                onClick = {},
            )
        }
    }
}
