/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.datechooser

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.DatePicker
import androidx.annotation.StringRes
import androidx.appcompat.app.AlertDialog
import androidx.core.view.isVisible
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentDateChooserBinding
import dk.nsp.mingraviditet.extensions.disable
import dk.nsp.mingraviditet.ui.widgets.BottomSheetViewModel
import dk.nsp.shared.util.extensions.toEpochMilli
import dk.nsp.util.textmodel.TextModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch
import java.time.LocalDate

@AndroidEntryPoint
class DateChooserFragment : Fragment() {
    private val vm: DateChooserViewModel by viewModels()
    private val bottomSheetViewModel: BottomSheetViewModel by activityViewModels()

    private var resetDateDialog: AlertDialog? = null

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val binding = FragmentDateChooserBinding.inflate(inflater, container, false)

        binding.confirmButton.setOnClickListener {
            binding.confirmButton.disable()
            vm.setDate()
        }

        binding.resetDate.setOnClickListener {
            resetDateDialog?.show()
        }

        binding.resetDateTextButton.setOnClickListener {
            resetDateDialog?.show()
        }

        binding.calculateDueDate.setOnClickListener {
            vm.changeToCalculateDueDate()
        }

        vm.onDateUpdated.observe(viewLifecycleOwner) {
            findNavController().popBackStack()
        }

        lifecycleScope.launch {
            vm.dateChooserDataFlow.collectLatest {
                val hasInitialDate = it.initialDate != null

                // Change thread to UI
                lifecycleScope.launch(Dispatchers.Main) {
                    resetDateDialog = createConfirmResetDateDialog(binding.root.context, it)

                    setupCalendarView(binding.calendar, minDate = it.minDate, maxDate = it.maxDate)

                    when (it) {
                        is DateChooserData.ExpectedDateOfDelivery -> {
                            bottomSheetViewModel.setTitle(TextModel.from(R.string.overview_pregnancy_select_expected_delivery_date_title))
                            if (hasInitialDate) {
                                binding.description.setText(R.string.overview_select_new_expected_delivery_date_description)
                                binding.confirmButton.setText(R.string.overview_pregnancy_select_new_expected_delivery_date_button_title)
                            } else {
                                binding.description.setText(R.string.overview_select_expected_delivery_date_description)
                                binding.confirmButton.setText(R.string.overview_pregnancy_select_expected_delivery_date_button_title)
                            }

                            binding.resetDateTextButton.isVisible = hasInitialDate
                            binding.calculateDueDate.isVisible = !hasInitialDate
                        }

                        is DateChooserData.DateOfBirth -> {
                            bottomSheetViewModel.setTitle(TextModel.from(R.string.overview_pick_date_of_birth_title))
                            binding.description.setText(R.string.overview_pick_date_of_birth_description)

                            binding.confirmButton.setText(R.string.overview_pick_date_of_birth_button)
                            binding.resetDate.isVisible = hasInitialDate
                        }

                        is DateChooserData.CalculateExpectedDateOfDelivery -> {
                            bottomSheetViewModel.setTitle(TextModel.from(R.string.overview_pregnancy_calculate_expected_delivery_date_title))
                            binding.description.setText(R.string.overview_select_calculated_expected_due_date_description)

                            binding.confirmButton.setText(R.string.overview_pregnancy_calculate_expected_delivery_date_button_title)
                            binding.calculateDueDate.isVisible = false
                        }
                    }
                }
            }
        }

        return binding.root
    }

    private fun setupCalendarView(calendar: DatePicker, minDate: LocalDate, maxDate: LocalDate) {
        // "Clear" dates so the view updates correctly
        calendar.minDate = LocalDate.now().toEpochMilli()
        calendar.maxDate = LocalDate.now().toEpochMilli()

        // Set actual min / max dates
        calendar.minDate = minDate.toEpochMilli()
        calendar.maxDate = maxDate.toEpochMilli()

        calendar.setOnDateChangedListener { _, year, monthOfYear, dayOfMonth ->
            vm.updateLocalDate(
                LocalDate.of(year, monthOfYear + 1, dayOfMonth),
            )
        }

        vm.pickedDate.observe(viewLifecycleOwner) {
            it?.let {
                if (it.year != calendar.year || it.monthValue != calendar.month - 1 || it.dayOfMonth != calendar.dayOfMonth) {
                    calendar.updateDate(it.year, it.monthValue - 1, it.dayOfMonth)
                }
            }
        }
    }

    private fun createConfirmResetDateDialog(context: Context, method: DateChooserData): AlertDialog {
        @StringRes val title: Int

        @StringRes val description: Int

        @StringRes val clearButton: Int

        when (method) {
            is DateChooserData.ExpectedDateOfDelivery -> {
                title = R.string.overview_reset_date_of_birth_alert_title
                description = R.string.overview_reset_expected_due_date_alert_description
                clearButton = R.string.common_delete
            }

            is DateChooserData.DateOfBirth -> {
                title = R.string.overview_reset_date_of_birth_alert_title
                description = R.string.overview_reset_date_of_birth_alert_description
                clearButton = R.string.common_cta_reset
            }

            // This is not a possibility at the moment, so strings are placeholders
            is DateChooserData.CalculateExpectedDateOfDelivery -> {
                title = R.string.overview_reset_date_of_birth_alert_title
                description = R.string.common_no
                clearButton = R.string.common_cta_reset
            }
        }

        return AlertDialog.Builder(context)
            .setTitle(title)
            .setMessage(description)
            .setPositiveButton(clearButton) { _, _ -> vm.clearDate() }
            .setNegativeButton(R.string.common_regret) { _, _ -> }
            .setCancelable(true)
            .create()
    }

    companion object {
        const val dateChooserMethodArgKey = "dateChooserMethodArgKey"
    }
}
