/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.datechooser

import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.asLiveData
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.PregnancyRepository
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.launch
import java.time.LocalDate
import javax.inject.Inject

sealed class DateChooserData(open val initialDate: LocalDate?, open val minDate: LocalDate, open val maxDate: LocalDate) {
    data class ExpectedDateOfDelivery(override val initialDate: LocalDate?, override val minDate: LocalDate, override val maxDate: LocalDate) :
        DateChooserData(initialDate, minDate, maxDate)

    data class DateOfBirth(override val initialDate: LocalDate?, override val minDate: LocalDate, override val maxDate: LocalDate) :
        DateChooserData(initialDate, minDate, maxDate)

    data class CalculateExpectedDateOfDelivery(override val minDate: LocalDate, override val maxDate: LocalDate) : DateChooserData(null, minDate, maxDate)
}

@HiltViewModel
class DateChooserViewModel @Inject constructor(
    savedStateHandle: SavedStateHandle,
    private val pregnancyRepository: PregnancyRepository,
) : ViewModel() {

    private val _method = MutableSharedFlow<DateChooserMethod>(replay = 1)

    init {
        val method = savedStateHandle.get<DateChooserMethod>(savedStateHandle.get<String>(DateChooserFragment.dateChooserMethodArgKey) ?: "")
        viewModelScope.launch {
            _method.emit(method ?: DateChooserMethod.ExpectedDateOfDelivery)
        }
    }

    private val pregnancy = pregnancyRepository.pregnancyInfo.map { it }

    private val initialDate = pregnancy.combine(_method) { pregnancyInfo, method ->
        when (method) {
            DateChooserMethod.ExpectedDateOfDelivery -> pregnancyInfo?.expectedDueDate
            DateChooserMethod.DateOfBirth -> pregnancyInfo?.dateOfBirth
            DateChooserMethod.CalculateExpectedDateOfDelivery -> null
        }
    }

    val dateChooserDataFlow = pregnancy.combine(_method) { pregnancyInfo, method ->
        when (method) {
            DateChooserMethod.ExpectedDateOfDelivery -> {
                DateChooserData.ExpectedDateOfDelivery(
                    initialDate = pregnancyInfo?.expectedDueDate,
                    minDate = LocalDate.now(),
                    maxDate = LocalDate.now().plusWeeks(40),
                )
            }

            DateChooserMethod.CalculateExpectedDateOfDelivery -> {
                DateChooserData.CalculateExpectedDateOfDelivery(
                    minDate = LocalDate.now().minusWeeks(40),
                    maxDate = LocalDate.now(),
                )
            }

            DateChooserMethod.DateOfBirth -> {
                if (pregnancyInfo == null) throw IllegalStateException("We should never be able to set date of birth, if pregnancy doesn't have a due date")

                DateChooserData.DateOfBirth(
                    initialDate = pregnancyInfo.dateOfBirth,
                    minDate = pregnancyInfo.dueDateMethod.lastMenstruationPeriod,
                    maxDate = LocalDate.now().coerceAtMost(pregnancyInfo.dueDateMethod.expectedDueDate.plusWeeks(2)),
                )
            }
        }
    }

    private val _pickedDate = MediatorLiveData<LocalDate>().apply {
        addSource(initialDate.asLiveData()) {
            postValue(it ?: LocalDate.now())
        }
    }

    val pickedDate: LiveData<LocalDate> = _pickedDate

    private val _onDateUpdated = MutableLiveData<Unit>()
    val onDateUpdated: LiveData<Unit> = _onDateUpdated

    fun changeToCalculateDueDate() = viewModelScope.launch {
        _method.emit(DateChooserMethod.CalculateExpectedDateOfDelivery)
    }

    fun updateLocalDate(dateOfBirth: LocalDate) {
        if (dateOfBirth != _pickedDate.value) {
            _pickedDate.postValue(dateOfBirth)
        }
    }

    fun setDate() = viewModelScope.launch {
        _method.collectLatest {
            when (it) {
                DateChooserMethod.ExpectedDateOfDelivery -> pregnancyRepository.setExpectedDueDate(_pickedDate.value)
                DateChooserMethod.DateOfBirth -> pregnancyRepository.setDateOfBirth(_pickedDate.value)
                DateChooserMethod.CalculateExpectedDateOfDelivery -> pregnancyRepository.setLastMenstruationPeriod(_pickedDate.value)
            }
            _onDateUpdated.postValue(Unit)
        }
    }

    fun clearDate() = viewModelScope.launch {
        _method.collectLatest {
            when (it) {
                DateChooserMethod.ExpectedDateOfDelivery -> pregnancyRepository.clearPregnancy()
                DateChooserMethod.DateOfBirth -> pregnancyRepository.setDateOfBirth(null)
                DateChooserMethod.CalculateExpectedDateOfDelivery -> Unit // N/A
            }
            _onDateUpdated.postValue(Unit)
        }
    }
}
